--[[ <Test>
 Execute this file,
 and run 'replace_with_upbeat(xx)'.
 xx is length of upbeat.
 example : 'replace_with_upbeat(1/4)'
--]]

local init_inst , init_inst_index
local init_sample_index , init_sample , init_buffer

local init_sample_rate , init_bit_depth
local init_number_of_channels , init_number_of_frames

local new_sample , new_buffer

local bpm, selection_start, selection_end



function get_init_env()
  init_inst = renoise.song().selected_instrument
  init_sample_index = renoise.song().selected_sample_index
  init_sample = renoise.song().selected_sample
  init_buffer = renoise.song().selected_sample.sample_buffer
  
  if init_buffer.has_sample_data ~= true
    then
      renoise.app():show_error("This has no sample data.")
    return nil
  end
  
  init_sample_rate =
       init_buffer.sample_rate
  init_bit_depth =
       init_buffer.bit_depth
  init_number_of_channels =
  init_buffer.number_of_channels
  init_number_of_frames =
  init_buffer.number_of_frames
  
  selection_start =
          init_buffer.selection_start
  selection_end =
          init_buffer.selection_end
  bpm = renoise.song().transport.bpm

end

       
function get_new_env()
--  init_inst:insert_sample_at(init_sample_index)
  new_sample =
          init_inst.samples[init_sample_index +1]
  
  new_buffer = new_sample.sample_buffer

end


function frames_per_xbeat (beat)
  bpm = renoise.song().transport.bpm
  local seconds_per_beat = 1 / bpm * 60
  return math.floor(seconds_per_beat * init_sample_rate * beat)
end


function upbeat_start_point(beat)
  selection_start =
          init_buffer.selection_start
  local s = (selection_start -1)
            - frames_per_xbeat(beat)
  if s >= 0 then
  return s+1
  else
  return s
  end
end


function frames_with_upbeat(upbeat)
  selection_start =
          init_buffer.selection_start
  selection_end =
          init_buffer.selection_end
  return
  frames_per_xbeat (upbeat)
  + selection_end
  - selection_start + 1
end


function create_new (num_frames)
  get_init_env()
  init_inst:insert_sample_at(init_sample_index+1)
  get_new_env()
       
  new_buffer:create_sample_data(
                 init_sample_rate,init_bit_depth,init_number_of_channels
                 , num_frames)
end

function create_with_upbeat (beat)
  local bool =
  create_new (
    frames_with_upbeat(beat))
  --[[for debug
  print('create new buffer:'..tostring(bool)) 
  --]] 
end



function set_new_with_upbeat (beat)
  create_with_upbeat (beat)
  
  local s = upbeat_start_point(beat)
  local new_num_frames = frames_with_upbeat(beat)
  
  --[[ for debug
   print("upbeat_frames:"..frames_per_xbeat (beat))
   print("selection_start:"..init_buffer.selection_start)
   print("selection_end:"..init_buffer.selection_end)
   print("s:"..s)
   print("new_num_frames:"..new_num_frames)
   print("num_frames_new:"..new_buffer.number_of_frames)
  --]] 

 new_buffer:prepare_sample_data_changes()
  
  if s > 0
    then          
      for ch = 1,init_number_of_channels do
        for fr = 1,new_num_frames do
          new_buffer
          :set_sample_data(ch,fr,
           init_buffer:sample_data(ch, fr+s-1))
        end
      end
    else
      
      for ch = 1,init_number_of_channels do
        for fr = 1,(-s) do
          new_buffer:set_sample_data(
            ch,fr,
            0)
        end
        for fr = (1-s),new_num_frames do
          new_buffer
          :set_sample_data(ch,fr,
          init_buffer
          :sample_data(ch,fr+s))
        end
      end
    end
 new_buffer:finalize_sample_data_changes()
end


function replace_with_upbeat (beat)
  set_new_with_upbeat (beat)
  init_sample:copy_from(new_sample)
  init_inst:delete_sample_at(init_sample_index + 1)
  init_buffer.selection_start = frames_per_xbeat(beat)+1
end


